#ifndef __norm_debug_h
#define __norm_debug_h

#include <stdio.h>
#include "norm_options.h"

#ifdef DEBUG
#define D(i, msg) \
{ \
  if (i <= options.debuglevel) \
    { \
      if (options.show_code_location) \
         printf("DBG [%s, %u %s()]: ", __FILE__, __LINE__, __FUNCTION__); \
      else \
         printf("[%s]: ", norm_options_get_dbg_id(i)); \
      printf msg; \
      fflush(stdout); \
    } \
}
#define D_ENTER { \
  nest_level++; \
  if (options.debuglevel >= NESTING_LOGLEVEL) \
    { \
      printf("ENTER  "); \
      whitespace(nest_level); \
      printf("%s, %u %s()\n", __FILE__, __LINE__, __FUNCTION__); \
      fflush(stdout); \
    } \
}
#define D_RETURN { \
  if (options.debuglevel >= NESTING_LOGLEVEL) \
    { \
      printf("RETURN "); \
      whitespace(nest_level); \
      printf("%s, %u %s()\n", __FILE__, __LINE__, __FUNCTION__); \
      fflush(stdout); \
    } \
  nest_level--; \
  return; \
}
#define D_RETURN_(x) \
{ \
  if (options.debuglevel >= NESTING_LOGLEVEL) \
    { \
      printf("RETURN "); \
      whitespace(nest_level); \
      printf("%s, %u %s()\n", __FILE__, __LINE__, __FUNCTION__); \
      fflush(stdout); \
    } \
  nest_level--; \
  return x; \
}
#define D_EXEC(i, x) \
{\
  if (i <= options.debuglevel) \
    { \
      x; \
    } \
}
#else
#define D(i, msg)
#define D_ENTER
#define D_RETURN \
{ \
  return; \
}
#define D_RETURN_(x) \
{ \
  return x; \
}
#define D_EXEC(i, x)
#endif

extern int nest_level;

void whitespace(int num);

#endif 

