#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <sys/time.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "norm.h"
#include "norm_debug.h"
#include "norm_misc.h"


int
norm_misc_ones_complement_checksum(const void *p, int b, u_int32_t sum)
{
  const u_short *sp;

  D_ENTER;

  sp = (u_short *) p; /* better be aligned! */
  b /= 2;             /* convert to count of short's */

  /* No need for endian conversions. */
  while ( --b >= 0 )
    sum += *sp++;

  while ( sum > 0xffff )
    sum = (sum & 0xffff) + (sum >> 16);

  D_RETURN_(sum);
}

int   
norm_misc_add_trusted_network(char *ip_addr, int maskbits)
{
  struct in_addr net;
  struct in_addr mask;

  if (options.num_trusted == NUM_TRUSTED)
    return -1;

  if (inet_aton(ip_addr, &net) > 0)
    {
      net.s_addr = htonl(net.s_addr);
      options.trusted_net[options.num_trusted] = net;

      mask.s_addr = ~( (2 << (32-maskbits-1)) - 1);

      /*
      mask.s_addr = 1;
      
      while ((net.s_addr & mask.s_addr) == 0)
	mask.s_addr = mask.s_addr*2 + 1;
      
      mask.s_addr = ~(mask.s_addr >> 1);
      */

      options.trusted_mask[options.num_trusted] = mask;
      options.num_trusted++;
    }

  return 0;
}


int   
norm_misc_add_network_interface(char *if_name)
{
  if (options.num_ifs == NUM_IFS)
    return -1;
  
  options.ifs[options.num_ifs] = if_name;
  options.num_ifs++;

  return 0;
}


int   
norm_misc_is_trusted_address(struct in_addr ip_orig)
{
  int             i;
  struct in_addr  ip;
  
  ip.s_addr = htonl(ip_orig.s_addr);  

  for(i = 0; i < options.num_trusted; i++)
    {      
      if ((ip.s_addr & options.trusted_mask[i].s_addr) -
	  options.trusted_net[i].s_addr == 0)
	return 1;
    }

  return 0;
}


