#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <sys/time.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "norm.h"
#include "norm_debug.h"
#include "norm_icmp.h"
#include "norm_ip.h"
#include "norm_ip_fragment.h"
#include "norm_misc.h"
#include "norm_tcp.h"
#include "norm_udp.h"
#include "norm_send.h"

/* packet_queue_element holds a queue of packets for sending */

struct packet_queue_element {
  NormPacket               p;
  Interface               *nif;
  struct packet_queue_element *next;
};

/*queue of normalizer-generated packets waiting to be sent*/
static struct packet_queue_element *packet_queue = NULL;

int
norm_packet_output(NormPacket *p, Interface *nif)
{
  D_ENTER;

  if (nif->dev)
    {
      /* We're live on a network -- write IP
	 packet including header to raw socket. */
      int cc;
      int length;
      struct ip *iphdr;

      iphdr = (struct ip*)p->network_data;
      iphdr->ip_len = htons(iphdr->ip_len);
      iphdr->ip_off = htons(iphdr->ip_off);
      iphdr->ip_sum = 0;
      iphdr->ip_sum = norm_ip_in_cksum((u_short*)iphdr, iphdr->ip_hl*4, 0);

      length = norm_packet_get_length_ip(p);

      cc = send(nif->s, p->network_data, length, 0);
      if (cc < 0 || cc != length)
	{
	  perror("problem in send");
	  D_RETURN_(-1);	}
    }
  else
    {
      /* Offline analysis -- write to tracefile.
      */
      if (p->ph)
	{
	  PCAP_DUMP((u_char*)nif->pcap_dumper, p->ph, p->link_data);
	}
      else
	{
	  /* pcap packet header does not exist -- create one. */
	  struct pcap_pkthdr pkthdr;
	  int                length;

	  gettimeofday(&(pkthdr.ts), NULL);

	  if (p->link_data)
	    {
	      length = norm_packet_get_length_link(p);
	      pkthdr.caplen = length;	
	      pkthdr.len = length;
	      PCAP_DUMP((u_char*)nif->pcap_dumper, &pkthdr, p->link_data);
	    }
	  else if (p->network_data)
	    {
	      length = norm_packet_get_length_ip(p);
	      pkthdr.caplen = length;	
	      pkthdr.len = length;
	      PCAP_DUMP((u_char*)nif->pcap_dumper, &pkthdr, p->network_data);
	    }
	  else
	    {
	      D(FATAL_LOGLEVEL, ("Strange packet for writing ...\n"));
	    }
	}
    }

  D_RETURN_(0);
}


int     
norm_packet_send(NormPacket *p, Interface *nif)
{
  D_ENTER;
  norm_packet_output(p, nif);
  D_RETURN_(norm_packet_flush_queue());
}


int
norm_packet_queue_for_sending(NormPacket *p, Interface *nif)
{
  struct packet_queue_element *qel;
  
  D_ENTER;

  qel = NEW(struct packet_queue_element);
  bzero(qel, sizeof(struct packet_queue_element));

  qel->next = packet_queue;
  qel->p.link_data = p->link_data;
  qel->p.network_data = p->network_data;
  qel->p.length  = p->length;
  qel->nif  = nif;

  packet_queue = qel;

  D_RETURN_(0);
}


int     
norm_packet_flush_queue(void)
{
  struct packet_queue_element *qel;

  D_ENTER;

  qel = packet_queue;
  
  while (qel)
    {
      norm_packet_output(&(qel->p), qel->nif);
      norm_empty_packet(&(qel->p));
      free(qel);
      qel = qel->next;
    }

  packet_queue = NULL;
  D_RETURN_(0);
}
